import { db, IRandomElementMixin } from "../db";
import { isAuthorised } from "../util";
import { ICommand } from "./command";
import { Collection } from "discord.js";
import { CollectionChain } from "lodash";

const quotePattern = /add quote by "([^"]+)"\s*(.*)/i;

function minify(str: string, maxLength: number) {
    let result = str.replace("\n", "");
    if (result.length > maxLength)
        result = `${result.substring(0, maxLength - 3)}...`;
    return result;
}

interface Quote {
    author: string,
    message: string
}

export default {
    commands: [
        {
            pattern: "add quote",
            action: (msg, c) => {
                if (!isAuthorised(msg.member))
                    return;
        
                let result = quotePattern.exec(c);
        
                if (result == null)
                    return;
        
                let author = result[1].trim();
                let message = result[2].trim();
        
                (db.get("quotes") as CollectionChain<any>).push({
                    author: author,
                    message: message
                }).write();
        
                msg.channel.send(`${msg.author.toString()} Added quote #${db.get("quotes").size().value()}!`);
            }
        },
        {
            pattern: "random quote",
            action: (msg) => {
                if (db.get("quotes").size().value() == 0) {
                    msg.channel.send("I have no quotes!");
                    return;
                }
                let quote = (db.get("quotes") as IRandomElementMixin).randomElement().value();
                let index = (db.get("quotes") as CollectionChain<any>).indexOf(quote).value();
                msg.channel.send(`Quote #${index + 1}:\n*"${quote.message}"*\n- ${quote.author}`);
            }
        },
        {
            pattern: "remove quote",
            action: (msg, c) => {
                let quoteNum = c.substring("remove quote".length).trim();
                let val = parseInt(quoteNum);
                if (isNaN(val) || db.get("quotes").size().value() < val - 1)
                    return;
        
                (db.get("quotes") as CollectionChain<any>).pullAt(val - 1).write();
                msg.channel.send(`${msg.author.toString()} Removed quote #${val}!`);
            }
        },
        {
            pattern: "quotes",
            action: msg => {
                if (!isAuthorised(msg.member)) {
                    msg.channel.send(`${msg.author.toString()} To prevent spamming, only bot moderators can view all quotes!`);
                    return;
                }
        
                if (db.get("quotes").size().value() == 0) {
                    msg.channel.send("I have no quotes!");
                    return;
                }
                
                let quotes = (db.get("quotes") as CollectionChain<any>).reduce((prev: string, curr: Quote, i: number) => `${prev}[${i+1}] "${minify(curr.message, 10)}" by ${curr.author}\n`, "\n").value();
                msg.channel.send(`${msg.author.toString()}I know the following quotes:\n\`\`\`${quotes}\`\`\``);
            }
        }
    ],
    documentation: {
        "add quote by \"<author>\" <NEWLINE> <quote>": {
            auth: true,
            description: "Adds a quote"
        },
        "remove quote <quote_index>": {
            auth: true,
            description: "Removes quote. Use \"quotes\" to get the <quote_index>!"
        },
        "quotes": {
            auth: true,
            description: "Lists all known quotes."
        },
        "random quote": {
            auth: false,
            description: "Shows a random quote by someone special..."
        }
    }
} as ICommand;


// module.exports = {
//     commands: commands,
//     documentation: documentation
// };